<?php

declare(strict_types=1);

namespace Drupal\entity_display_route\Controller;

use Drupal\Core\DependencyInjection\AutowireTrait;
use Drupal\Core\Entity\Controller\EntityViewController as BaseEntityViewController;
use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\entity_display_route\EntityDisplayRouteInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Defines an entity view controller that allows per-bundle opt-out.
 */
final class EntityViewController extends BaseEntityViewController {

  use AutowireTrait;

  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    RendererInterface $renderer,
    protected readonly EntityDisplayRepositoryInterface $entityDisplayRepository,
  ) {
    parent::__construct($entity_type_manager, $renderer);
  }

  /**
   * {@inheritdoc}
   */
  public function view(EntityInterface $_entity, $view_mode = 'full'): array {
    $display = $this->entityDisplayRepository->getViewDisplay($_entity->getEntityTypeId(), $_entity->bundle(), $view_mode);
    if ($display->getThirdPartySetting(EntityDisplayRouteInterface::MODULE, EntityDisplayRouteInterface::HIDE_PAGE, FALSE)) {
      throw new NotFoundHttpException();
    }
    return parent::view($_entity, $view_mode);
  }

}
