<?php

declare(strict_types=1);

namespace Drupal\entity_display_route\Routing;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Routing\RouteSubscriberBase;
use Drupal\entity_display_route\EntityDisplayRouteInterface;
use Symfony\Component\Routing\Route;
use Symfony\Component\Routing\RouteCollection;

/**
 * Defines a route subscriber to add additional routes.
 */
final class RouteSubscriber extends RouteSubscriberBase {

  public function __construct(
    protected EntityTypeManagerInterface $entityTypeManager,
  ) {
  }

  /**
   * {@inheritdoc}
   */
  protected function alterRoutes(RouteCollection $collection) {
    $modes = $this->entityTypeManager->getStorage('entity_view_mode')->loadMultiple(NULL);
    /** @var \Drupal\Core\Entity\EntityViewModeInterface $mode */
    foreach ($modes as $id => $mode) {
      $path = $mode->getThirdPartySetting(EntityDisplayRouteInterface::MODULE, EntityDisplayRouteInterface::PATH, '');
      if ($path === '') {
        continue;
      }
      [, $display_id] = \explode('.', $id);
      $entity_type_id = $mode->getTargetType();
      $entity_type = $this->entityTypeManager->getDefinition($entity_type_id);
      $template = $entity_type->getLinkTemplate('canonical');
      if ($template === FALSE) {
        continue;
      }
      $route = new Route(\sprintf('%s/%s', $template, $path));
      $route
        ->addDefaults([
          '_entity_view' => \sprintf('%s.%s', $entity_type_id, $display_id),
          '_title_callback' => '\Drupal\Core\Entity\Controller\EntityController::title',
        ])
        ->setRequirement('_entity_access', \sprintf('%s.view', $entity_type_id))
        ->setOption('parameters', [
          $entity_type_id => ['type' => 'entity:' . $entity_type_id],
        ]);

      $canonical = $collection->get(\sprintf('entity:%s:canonical', $entity_type_id));
      if ($canonical !== NULL) {
        $requirement = $route->getRequirement($entity_type_id);
        if ($requirement !== NULL) {
          $route->setRequirement($entity_type_id, $requirement);
        }
      }
      $collection->add(\sprintf('entity.%s.entity_view_display__%s', $entity_type_id, $display_id), $route);
    }
    // Node has its own RouteProvider for the canonical view, but it just ends
    // up calling the parent method anyway.
    if ($route = $collection->get('entity.node.canonical')) {
      $defaults = $route->getDefaults();
      $defaults['_entity_view'] = 'node.full';
      unset($defaults['_controller']);
      $route->setDefaults($defaults);
    }
  }

}
